import os
from os.path import split
import re

keys = ['title', 'rating', 'writer', 'translator', 'pub_info', 'ISBN', 'series', 'tag']
type_template = "<http://www.w3.org/2001/XMLSchema#{}>"
book_p_template = "<http://kg.com/book/{:06d}> <http://kg.com/book/ontology/{}> \"{}\"^^<http://www.w3.org/2001/XMLSchema#{}> ."
book_r_template = "<http://kg.com/book/{:06d}> <http://kg.com/book/ontology/{}> <http://kg.com/{}/{:06d}> ."
writer_template = "<http://kg.com/person/{:06d}> <http://kg.com/book/ontology/{}> \"{}\"^^<http://www.w3.org/2001/XMLSchema#string> ."

books = []
person2id = {}
person_id = 0  # person has multi names, can not use len(perison2id) to get new id

country_person = {}
pid2chn = {}
pid2eng = {}
press_set = set()
tag_set = set()

country_dict = {}
with open('./processed/countries.txt') as f:
    for line in f:
        line = line.strip()
        name, full_name = line.strip().split('\t')
        country_dict[name] = full_name
        country_person[full_name] = set()
country_person['None'] = set()

def get_writer(text):
    
    def clean_name(name, verbose = ['编', '绘', '图', '主编', '摄影', '序', '译' , '著', '|译者', '编著' '导读', '出品']):
        if name == None or name.strip() == '':
            return name
        for v in verbose:
            if name.endswith(v):
                name = name[:name.find(v)]
                break
        name = name.strip()
        if name.endswith('等'):
            name = name[:-1]
        return name
        
    country, chinese_name, eng_name = None, None, None
    text = text.strip()
    if len(text) == 0:
        return country, chinese_name, eng_name
    if text[0] in ['（', '(', '[', '【']:
        right = -1
        if text[0] == '（':
            right = text.find('）')
        elif text[0] == '(':
            right = text.find(')')
            if right == -1:
                right = text.find('）') # special case!!
        elif text[0] == '[':
            right = text.find(']')
            if right == -1:
                right = text.find('］') # special case!!
            if right == -1:
                right = text.find('}') # special case!!
        else:
            right = text.find('】')
        if right == -1:
            print('==> Error when parsing writer: {}'.format(text))
            return country, chinese_name, eng_name
        country_name = text[1:right].strip()
        if country_name in country_dict:
            country = country_dict[country_name]
        text = text[right+1:].strip()
    rules = [r'(.{1,})\((.{1,})\)', r'(.{1,})（(.{1,})）', r'\((.{1,})\)(.{1,})', r'（(.{1,})）(.{1,})']
    search = None
    for r in rules:
        search = re.search(r, text)
        if search is not None:
            break
    if search is not None:
        name1, name2 = search.group(1), search.group(2)
        if name1[0].encode( 'UTF-8' ).isalpha():
            chinese_name, eng_name = name2, name1
        else:
            chinese_name, eng_name = name1, name2
    else:
        if text[0].encode( 'UTF-8' ).isalpha():
            eng_name = text
        else:
            chinese_name = text
    chinese_name = clean_name(chinese_name)
    if chinese_name is not None:
        chinese_name = chinese_name.replace(" ", "")
    if chinese_name == '史密斯(Smith':
        print(chinese_name)
    eng_name = clean_name(eng_name)
    return country, chinese_name, eng_name


def convert(text): # 处理转义特殊字符
    return text.replace("\\", "\\\\").replace("\"", '\\\"')

def convert_date(text): # get valid date
    # print(text)
    if text.find('-') == -1:  # like 201006 or 20100318
        if len(text) == 6:
            text = text[:4] + '-' + text[4:6] + '-01'
        else:
            text = text[:4] + '-' + text[4:6] + '-' + text[6:]   
    else:
        lst = text.split('-')
        if len(lst) == 2: # like 2010-06
            lst.append('01')
        lst = [int(i) for i in lst]
        text = '{:04d}-{:02d}-{:02d}'.format(*lst)
    # print(text)
    assert re.search(r'^\d{4}-\d{2}-\d{2}$', text) is not None
    return text
def convert_to_ntriple(text):
    global person_id

    ntriples = []
    text = text.strip()
    search = re.search(r'title:(.*) rating:(.*) writer:(.*) translator:(.*) pub_info:(.*) ISBN:(.*) series:(.*) tags:(.*)', text)
    if search is None:
        print('Unrecognized book format!', text)
        return ntriples
    
    title = search.group(1).strip()
    if len(title) == 0:
        return ntriples
    # if title in book2id:
    #     b_id = book2id[title]
    # else:
    #     b_id = len(book2id) + 1
    #     book2id[title] = b_id
    #     ntriples.append(book_p_template.format(b_id, 'title', convert(title), 'string'))
    # 同一本书在豆瓣中可能会有多个信息略有差别的词条，就当他们是多个实体吧:(
    b_id = len(books) + 1
    books.append(title)
    ntriples.append(book_p_template.format(b_id, 'title', convert(title), 'string'))

    writer_text = ''
    for i in range(2, len(keys)+1):
        value = search.group(i).strip()
        if value.replace(' ', '') == 'None' or len(value.replace(' ', '')) == 0:
            continue
        key = keys[i-1]
        if key == 'rating':
            ntriples.append(book_p_template.format(b_id, key, value, 'float'))
        elif key == 'writer':
            writer_text = value
            writers = re.split(r'[/\\；;，、]', value)
            for writer in writers:
                country, chinese_name, eng_name = get_writer(writer)
                if chinese_name is None and eng_name is None:
                    # print(value)
                    continue
                if chinese_name not in person2id and eng_name not in person2id:
                    p_id = person_id + 1
                    person_id += 1
                    if chinese_name is not None:
                        ntriples.append(writer_template.format(p_id, 'chineseName', chinese_name))
                        person2id[chinese_name] = p_id
                        pid2chn[p_id] = chinese_name
                    if eng_name is not None:
                        ntriples.append(writer_template.format(p_id, 'engName', eng_name))
                        person2id[eng_name] = p_id
                        pid2eng[p_id] = eng_name
                else:
                    if chinese_name not in person2id and chinese_name is not None:
                        ntriples.append(writer_template.format(person2id[eng_name], 'chineseName', chinese_name))
                        person2id[chinese_name] = person2id[eng_name]
                        pid2chn[person2id[eng_name]] = chinese_name
                    if eng_name not in person2id and eng_name is not None:
                        ntriples.append(writer_template.format(person2id[chinese_name], 'engName', eng_name))
                        person2id[eng_name] = person2id[chinese_name]
                        pid2eng[person2id[chinese_name]] = eng_name
                
                if chinese_name is not None:
                    p_id = person2id[chinese_name]
                else:
                    p_id = person2id[eng_name]
                if country is None:
                    country = 'None'
                if p_id not in country_person[country]:
                    country_person[country].add(p_id)
                    if country != 'None':
                        ntriples.append(writer_template.format(p_id, 'country', country))
                    
                ntriples.append(book_r_template.format(b_id, key, 'person', p_id))
        elif key == 'translator':
            translators = re.split(r'[/\\；;，,、]', value)
            for t in translators:
                t = t.replace(' ', '')
                if t not in person2id:
                    p_id = person_id + 1
                    person_id += 1
                    person2id[t] = p_id
                    country_person['中国'].add(p_id)
                    ntriples.append(writer_template.format(p_id, 'chineseName', t))
                    ntriples.append(writer_template.format(p_id, 'country', '中国'))
                    pid2chn[p_id] = t
                p_id = person2id[t]
                ntriples.append(book_r_template.format(b_id, key, 'person', p_id))
        elif key == 'tag':
            tags = re.split(r'[/\\；;，,、\s]',value)
            for tag in tags:
                tag = tag.replace(' ', '')
                if len(tag) == 0:
                    continue
                ntriples.append(book_p_template.format(b_id, key, convert(tag), 'string'))
                tag_set.add(tag)
        elif key == 'pub_info':
            value = value.replace(' ', '')
            pub_info = value.split('/')
            rules = {'date': (r"\d+\-\d+(\-\d+)?|\d{6,8}", 'date'), 'price': (r"\d+(\.\d+)?", 'float')}
            for info in pub_info:
                match = False
                for key, (rule, type) in rules.items():
                    temp = re.search(rule, info)
                    if temp is not None:
                        v = temp.group()
                        # if key == 'date' and v.find('-') == -1:
                        #     if len(v) == 6:
                        #         v = v[:4] + '-' + v[4:] 
                        #     elif len(v) > 6:
                        #         v = v[:4] + '-' + v[4:6] + '-' + v[6:]  
                        if key == 'date':
                            v = convert_date(v)
                        ntriples.append(book_p_template.format(b_id, key, v, type))
                        match = True
                        break
                if not match:
                    if info != writer_text:
                        ntriples.append(book_p_template.format(b_id, 'press', convert(info), 'string'))
                        press_set.add(info)

        else: # 'ISBN', 'series'
            value = value.replace(' ', '')
            ntriples.append(book_p_template.format(b_id, key, convert(value), 'string'))
    return ntriples         

        

data_dir = './books'
output_file = './processed/ntriples.nt'
book_file = './processed/books.txt'
person_file = './processed/person.txt'
tag_file = './processed/tags.txt'
press_file = './processed/press.txt'
source_files = os.listdir(data_dir)
ntriples = []
for file in source_files:
    for line in open(os.path.join(data_dir, file), encoding='UTF-8'):
        ntriples.extend(convert_to_ntriple(line))


# N-Triples
print(f'==> Info: import {len(ntriples)} N-Triples!')
with open(output_file, 'w') as f:
    for item in ntriples:
        print(item, file=f)
print(f'==> Info: N-Triples write to {output_file}!')
# books
print(f'==> Info: found {len(books)} Books!')
with open(book_file, 'w') as f:
    for k, book in enumerate(books):
        print('{}\t{:06d}'.format(book, k+1), file=f)
print(f'==> Info: Books write to {book_file}!') 
# person 
print(f'==> Info: found {person_id} people(writer or translator)!')
with open(person_file, 'w') as f:
    for c, p_ids in country_person.items():
        for pid in p_ids:
            print('{}\t{}\t{}\t{:06d}'.format(c, pid2chn[pid] if pid in pid2chn else 'None', pid2eng[pid] if pid in pid2eng else 'None', pid), file=f)
print(f'==> Info: Persons write to {person_file}!')  
# tags
print(f'==> Info: found {len(tag_set)} tags!')
with open(tag_file, 'w') as f:
    for tag in tag_set:
        print(tag, file=f)
print(f'==> Info: Tags write to {tag_file}!')  
# press
print(f'==> Info: found {len(press_set)} presses!')
with open(press_file, 'w') as f:
    for press in press_set:
        print(press, file=f)
print(f'==> Info: Tags write to {press_file}!')  


