# encoding=utf-8

"""

@author: SimmerChan

@contact: hsl7698590@gmail.com

@file: query_main.py

@time: 2017/12/20 15:29

@desc:main函数，整合整个处理流程。

"""
import os
import sys
import jena_sparql_endpoint
import question2sparql

class Logger(object):
    def __init__(self, fpath=None):
        self.console = sys.stdout
        self.file = None
        if fpath is not None:
            self.file = open(fpath, 'w')

    def __del__(self):
        self.close()

    def __enter__(self):
        pass

    def __exit__(self, *args):
        self.close()

    def write(self, msg):
        self.console.write(msg)
        if self.file is not None:
            self.file.write(msg)

    def flush(self):
        self.console.flush()
        if self.file is not None:
            self.file.flush()
            os.fsync(self.file.fileno())

    def close(self):
        self.console.close()
        if self.file is not None:
            self.file.close()

if __name__ == '__main__':
    # 打印log文件
    sys.stdout = Logger('log.txt')
    # TODO 连接Fuseki服务器。
    fuseki = jena_sparql_endpoint.JenaFuseki()
    # TODO 初始化自然语言到SPARQL查询的模块，参数是外部词典列表。
    q2s = question2sparql.Question2Sparql(
        ['./dicts/book_dict.txt',
            './dicts/person_dict.txt',
            './dicts/press_dict.txt',
            './dicts/tags_dict.txt'])

    while True:
        question = input('>>> ')
        my_query = q2s.get_sparql(question)
        print('\n==> 查询语句:\n{}'.format(my_query))
        print('==> 查询结果:')
        if my_query is not None:
            result = fuseki.get_sparql_result(my_query)
            value = fuseki.get_sparql_result_value(result)

            # TODO 判断结果是否是布尔值，是布尔值则提问类型是"ASK"，回答“是”或者“不是”。
            if isinstance(value, bool):
                if value is True:
                    print('是')
                else:
                    print('不是')  # 可能是由于不存在这条知识而回答不是
            else:
                # TODO 查询结果为空，根据OWA，回答“不知道”
                if len(value) == 0:
                    print('I don\'t know. :(')
                elif len(value) == 1:
                    print(value[0])
                else:
                    output = ''
                    for v in value:
                        output += v + u'、'
                    print(output[0:-1])

        else:
            # TODO 自然语言问题无法匹配到已有的正则模板上，回答“无法理解”
            print('I can\'t understand. :(')

        print('\nquestion: {}'.format(question))
        print('\n' + '#' * 100 + '\n')
