# encoding=utf-8

"""

@author: SimmerChan

@contact: hsl7698590@gmail.com

@file: question_temp.py

@time: 2017/12/20 15:30

@desc:
设置问题模板，为每个模板设置对应的SPARQL语句
"""

from refo import finditer, Predicate, Star, Any, Disjunction
import re

# TODO SPARQL前缀和模板
SPARQL_PREXIX = u"""
PREFIX : <http://kg.com/book/ontology/>
PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>
"""

SPARQL_SELECT_TEM = u"{prefix}\n" + \
    u"SELECT DISTINCT {select} WHERE {{\n" + \
    u"{expression}\n" + \
    u"}}\n"

SPARQL_SELECT_TEM_ORDER = u"{prefix}\n" + \
    u"SELECT DISTINCT {select} WHERE {{\n" + \
    u"{expression}\n" + \
    u"}}\nORDER BY DESC({sort})"

SPARQL_COUNT_TEM = u"{prefix}\n" + \
    u"SELECT COUNT({select}) WHERE {{\n" + \
    u"{expression}\n" + \
    u"}}\n"

SPARQL_ASK_TEM = u"{prefix}\n" + \
    u"ASK {{\n" + \
    u"{expression}\n" + \
    u"}}\n"

INDENT = "    "


class W(Predicate):
    def __init__(self, token=".*", pos=".*"):
        self.token = re.compile(token + "$")
        self.pos = re.compile(pos + "$")
        super(W, self).__init__(self.match)

    def match(self, word):
        m1 = self.token.match(word.token)
        m2 = self.pos.match(word.pos)
        return m1 and m2


class Rule(object):
    def __init__(self, condition_num, condition=None, action=None):
        assert condition and action
        self.condition = condition
        self.action = action
        self.condition_num = condition_num

    def apply(self, sentence):
        matches = []
        for m in finditer(self.condition, sentence):
            i, j = m.span()
            matches.extend(sentence[i:j])

        return self.action(matches), self.condition_num


class KeywordRule(object):
    def __init__(self, condition=None, action=None):
        assert condition and action
        self.condition = condition
        self.action = action

    def apply(self, sentence):
        matches = []
        for m in finditer(self.condition, sentence):
            i, j = m.span()
            matches.extend(sentence[i:j])
        if len(matches) == 0:
            return None
        else:
            return self.action()

# TODO 定义关键词
pos_person = "nr"
pos_book = "nw"
pos_number = "m"
pos_place = "ns"
pos_time = "t"
pos_org = ["nt", "ORG"]

person_entity = (W(pos=pos_person))
book_entity = (W(pos=pos_book))
number_entity = (W(pos=pos_number))
time_entity =  (W(pos=pos_time))
place_entity = (W(pos=pos_place))
press_entity = (W(pos="nt") | W(pos="ORG"))
numeric_entity = time_entity | number_entity


translate = W("翻译")
write = (W("写")| W("出版") | W("有"))
press_v = W("出版")

book = (W("书") | W("书籍") | W("书本"))
writer = (W("作者")| W("作家"))
translator =(W("译者")| W("翻译"))

press = (W("出版社")| W("出版公司"))
rating = (W("评分") | W("分") | W("分数"))
price = (W("价格")| W("多少钱") | W("多贵") | W("售价") | W("标价"))
date =  (W("出版时间")| W("发行时间"))
isbn = (W("ISBN")| W("isbn"))
series = W("系列")
tag = (W("类型") | W("标签") | W("分类"))
book_basic = (press | rating | price | date | isbn | series | tag)

country = (W("国家") | W("国籍") | W("哪") + W("国人") | W("哪国") + W("人") | W("哪国人"))
english_name = (W("英文名") | W("英文") + W("名字") | W("外文") + W("名") | W("外文") + W("名字"))
chinese_name = (W("中文名") | W("中文") + W("名字") | W("中文") + W("名") | W("中文") + W("名字"))
person_basic = (country | english_name | chinese_name) 

higher = (W("大于") | W("高于") | W("高") | W("迟") | W("贵"))
lower = (W("小于") | W("低于")| W("低") | W("早") | W("便宜"))
equal = (W("等于") | W("刚好"))
compare = (higher | lower | equal)

several = (W("多少") | W("几"))
what = W("什么")
when = (W("何时") | W("时候") | W("时间"))
who = (W("谁")| W("哪个人"))
where = (W("哪里") | W("哪儿") | W("何地") | W("何处") | W("在") + W("哪"))
bool_flag = W("吗")

class QuestionSet:
    def __init__(self):
        pass

    @staticmethod
    def write_book_question(word_objects):
        """
        某个人写了什么书
        :param word_objects:
        :return:
        """
        select = u"?title"

        sparql = None
        for w in word_objects:
            if w.pos == pos_person:
                if w.token[0].encode("UTF-8").isalpha():
                    e = u"?p :engName '{person}'." \
                    u"?b :title ?title ." \
                    u"?b :writer ?p".format(person=w.token)
                else:
                    e = u"?p :chineseName '{person}'." \
                        u"?b :title ?title ." \
                        u"?b :writer ?p".format(person=w.token)

                sparql = SPARQL_SELECT_TEM.format(prefix=SPARQL_PREXIX,
                                                  select=select,
                                                  expression=e)
                break
        return sparql
    
    @staticmethod
    def translate_book_question(word_objects):
        """
        某个人翻译了什么书
        :param word_objects:
        :return:
        """
        select = u"?title"

        sparql = None
        for w in word_objects:
            if w.pos == pos_person:
                if w.token[0].encode("UTF-8").isalpha():
                    e = u"?p :engName '{person}'." \
                    u"?b :title ?title ." \
                    u"?b :translator ?p".format(person=w.token)
                else:
                    e = u"?p :chineseName '{person}'." \
                        u"?b :title ?title ." \
                        u"?b :translator ?p".format(person=w.token)

                sparql = SPARQL_SELECT_TEM.format(prefix=SPARQL_PREXIX,
                                                  select=select,
                                                  expression=e)
                break
        return sparql

    @staticmethod
    def has_writer_question(word_objects):
        """
        某本书的作者是谁
        :param word_objects:
        :return:
        """
        select = u"?x"

        sparql = None
        for w in word_objects:
            if w.pos == pos_book:
                e = u"?b :title '{book}'." \
                    u"?b :writer ?p." \
                    u"?p :chineseName ?x".format(book=w.token)

                sparql = SPARQL_SELECT_TEM.format(prefix=SPARQL_PREXIX,
                                                  select=select,
                                                  expression=e)
                break

        return sparql
    
    @staticmethod
    def has_translator_question(word_objects):
        """
        某本书的译者是谁
        :param word_objects:
        :return:
        """
        select = u"?x"

        sparql = None
        for w in word_objects:
            if w.pos == pos_book:
                e = u"?b :title '{book}'." \
                    u"?b :translator ?p." \
                    u"?p :chineseName ?x".format(book=w.token)

                sparql = SPARQL_SELECT_TEM.format(prefix=SPARQL_PREXIX,
                                                  select=select,
                                                  expression=e)
                break

        return sparql
    
    @staticmethod
    def has_basic_person_info_question(word_objects):
        """
        某作者/译者的基本信息是什么
        :param word_objects:
        :return:
        """

        keyword = None
        for r in person_basic_keyword_rules:
            keyword = r.apply(word_objects)
            if keyword is not None:
                break

        select = u"?x"
        sparql = None
        for w in word_objects:
            if w.pos == pos_person:
                if w.token[0].encode("UTF-8").isalpha():
                    e = u"?s :engName '{person}'." \
                        u"?s {keyword} ?x.".format(person=w.token, keyword=keyword)
                else:
                    e = u"?s :chineseName '{person}'." \
                        u"?s {keyword} ?x.".format(person=w.token, keyword=keyword)
                sparql = SPARQL_SELECT_TEM.format(
                    prefix=SPARQL_PREXIX, select=select, expression=e)

                break

        return sparql

    @staticmethod
    def has_basic_book_info_question(word_objects):
        """
        某书籍的基本信息是什么
        :param word_objects:
        :return:
        """

        keyword = None
        for r in book_basic_keyword_rules:
            keyword = r.apply(word_objects)
            if keyword is not None:
                break

        select = u"?x"
        sparql = None
        for w in word_objects:
            if w.pos == pos_book:
                e = u"?s :title '{book}'." \
                    u"?s {keyword} ?x.".format(book=w.token, keyword=keyword)

                sparql = SPARQL_SELECT_TEM.format(
                    prefix=SPARQL_PREXIX, select=select, expression=e)

                break

        return sparql

    @staticmethod
    def has_book_time_question(word_objects):
        """
        某本书是什么时候出版的
        :param word_objects:
        :return:
        """
        select = u"?x"

        sparql = None
        for w in word_objects:
            if w.pos == pos_book:
                e = u"?b :title '{book}'." \
                    u"?b :date ?x".format(book=w.token)

                sparql = SPARQL_SELECT_TEM.format(prefix=SPARQL_PREXIX,
                                                  select=select,
                                                  expression=e)
                break

        return sparql

    @staticmethod
    def has_compare_writer_question(word_objects):
        """
        某作者写的评分高于X（价格比X贵/出版时间比X晚）的书有哪些？
        :param word_objects:
        :return:
        """
        select = u"?x ?r"

        person = None
        number = None
        keyword = None

        for r in compare_keyword_rules:
            keyword = r.apply(word_objects)
            if keyword is not None:
                break
        
        relation = None
        for r in book_basic_keyword_rules:
            relation = r.apply(word_objects)
            if relation is not None:
                break

        for w in word_objects:
            if w.pos == pos_person:
                person = w.token

            if w.pos == pos_number:
                number = w.token

            if w.pos == pos_time:
                number = w.token + "^^xsd:date"

        if person is not None and number is not None and keyword is not None and relation is not None:
            if person[0].encode("UTF-8").isalpha():
                e = u"?p :engName '{person}'." \
                    u"?b :writer ?p." \
                    u"?b :title ?x." \
                    u"?b {relation} ?r." \
                    u"filter(?r {mark} {number})".format(person=person, relation = relation, number=number,
                                                        mark=keyword)
            else:
                 e = u"?p :chineseName '{person}'." \
                    u"?b :writer ?p." \
                    u"?b :title ?x." \
                    u"?b {relation} ?r." \
                    u"filter(?r {mark} {number})".format(person=person, relation = relation, number=number,
                                                        mark=keyword)
            return SPARQL_SELECT_TEM_ORDER.format(prefix=SPARQL_PREXIX,
                                            select=select,
                                            expression=e,
                                            sort='?r')
        else:
            return None

    @staticmethod
    def has_compare_translator_question(word_objects):
        """
        某人翻译的评分高于X（价格比X贵/出版时间比X晚）的书有哪些？
        :param word_objects:
        :return:
        """
        select = u"?x ?r"

        person = None
        number = None
        keyword = None

        for r in compare_keyword_rules:
            keyword = r.apply(word_objects)
            if keyword is not None:
                break
        
        relation = None
        for r in book_basic_keyword_rules:
            relation = r.apply(word_objects)
            if relation is not None:
                break

        for w in word_objects:
            if w.pos == pos_person:
                person = w.token

            if w.pos == pos_number:
                number = w.token

            if w.pos == pos_time:
                number = w.token + "^^xsd:date"

        if person is not None and number is not None and keyword is not None and relation is not None:
            if person[0].encode("UTF-8").isalpha():
                e = u"?p :engName '{person}'." \
                    u"?b :translator ?p." \
                    u"?b :title ?x." \
                    u"?b {relation} ?r." \
                    u"filter(?r {mark} {number})".format(person=person, relation = relation, number=number,
                                                        mark=keyword)
            else:
                 e = u"?p :chineseName '{person}'." \
                    u"?b :translator ?p." \
                    u"?b :title ?x." \
                    u"?b {relation} ?r." \
                    u"filter(?r {mark} {number})".format(person=person, relation = relation, number=number,
                                                        mark=keyword)
            return SPARQL_SELECT_TEM_ORDER.format(prefix=SPARQL_PREXIX,
                                            select=select,
                                            expression=e,
                                            sort="?r")
        else:
            return None
    
    @staticmethod
    def has_compare_question(word_objects):
        """
        评分高于X（价格比X贵/出版时间比X晚）的书有哪些？
        :param word_objects:
        :return:
        """
        select = u"?x ?r"

        number = None
        keyword = None

        for r in compare_keyword_rules:
            keyword = r.apply(word_objects)
            if keyword is not None:
                break
        
        relation = None
        for r in book_basic_keyword_rules:
            relation = r.apply(word_objects)
            if relation is not None:
                break

        for w in word_objects:

            if w.pos == pos_number:
                number = w.token

            if w.pos == pos_time:
                number = w.token + "^^xsd:date"

        if number is not None and keyword is not None and relation is not None:
            e = u"?b :title ?x." \
                u"?b {relation} ?r." \
                u"filter(?r {mark} {number})".format(relation = relation, number=number,
                                                    mark=keyword)
            return SPARQL_SELECT_TEM_ORDER.format(prefix=SPARQL_PREXIX,
                                            select=select,
                                            expression=e,
                                            sort="?r")
        else:
            return None

    @staticmethod
    def has_book_tag_question(word_objects):
        """
        某个标签的书有哪些
        :param word_objects:
        :return:
        """
        select = u"?x"

        sparql = None
        for w in word_objects:
            if w.pos == 'nz':
                e = u"?s :title ?x." \
                    u"?s :tag '{tag}'.".format(tag=w.token)

                sparql = SPARQL_SELECT_TEM.format(prefix=SPARQL_PREXIX,
                                                  select=select,
                                                  expression=e)
                break
        return sparql
    
    @staticmethod
    def has_book_isbn_question(word_objects):
        """
        ISBN号为x的书是
        :param word_objects:
        :return:
        """
        select = u"?x"

        sparql = None
        for w in word_objects:
            if w.pos == pos_number:
                e = u"?s :title ?x." \
                    u"?s :ISBN '{isbn}'.".format(isbn=w.token)

                sparql = SPARQL_SELECT_TEM.format(prefix=SPARQL_PREXIX,
                                                  select=select,
                                                  expression=e)
                break
        return sparql
    
    @staticmethod
    def has_book_press_question(word_objects):
        """
        出版社为x的书有哪些
        :param word_objects:
        :return:
        """
        select = u"?x"

        sparql = None
        for w in word_objects:
            if w.pos in pos_org:
                e = u"?s :title ?x." \
                    u"?s :press '{press}'.".format(press=w.token)

                sparql = SPARQL_SELECT_TEM.format(prefix=SPARQL_PREXIX,
                                                  select=select,
                                                  expression=e)
                break
        return sparql

    @staticmethod
    def has_write_book_quantity_question(word_objects):
        """
        某作者写了多少本书
        :param word_objects:
        :return:
        """
        select = u"?x"

        sparql = None
        for w in word_objects:
            if w.pos == pos_person:
                if w.token[0].encode('UTF-8').isalpha():
                    e = u"?p :engName '{person}'." \
                        u"?x :writer ?p.".format(person=w.token)  
                else:  
                    e = u"?p :chineseName '{person}'." \
                        u"?x :writer ?p.".format(person=w.token)

                sparql = SPARQL_COUNT_TEM.format(
                    prefix=SPARQL_PREXIX, select=select, expression=e)
                break

        return sparql

    @staticmethod
    def has_translate_book_quantity_question(word_objects):
        """
        某作者翻译了多少本书
        :param word_objects:
        :return:
        """
        select = u"?x"

        sparql = None
        for w in word_objects:
            if w.pos == pos_person:
                if w.token[0].encode('UTF-8').isalpha():
                    e = u"?p :engName '{person}'." \
                        u"?x :translator ?p.".format(person=w.token)  
                else:  
                    e = u"?p :chineseName '{person}'." \
                        u"?x :translator ?p.".format(person=w.token)

                sparql = SPARQL_COUNT_TEM.format(
                    prefix=SPARQL_PREXIX, select=select, expression=e)
                break

        return sparql

    @staticmethod
    def is_writer_question(word_objects):
        """
        xx是xx写的吗/xx是xx的作者吗
        :param word_objects:
        :return:
        """
        sparql = None

        person = None
        book = None
        for w in word_objects:
            if w.pos == pos_person:
                person = w.token
            if w.pos == pos_book:
                book = w.token
        if person is not None and book is not None:
            if person[0].encode('UTF-8').isalpha():
                e = u"?p :engName '{person}'." \
                    u"?b :title '{book}'." \
                    u"?b :writer ?p.".format(person=person, book=book)  
            else:  
                e = u"?p :chineseName '{person}'." \
                    u"?b :title '{book}'." \
                    u"?b :writer ?p.".format(person=person, book=book)

            sparql = SPARQL_ASK_TEM.format(
                prefix=SPARQL_PREXIX, expression=e)

        return sparql

    @staticmethod
    def is_tanslator_question(word_objects):
        """
        xx是xx翻译的吗
        :param word_objects:
        :return:
        """
        sparql = None

        person = None
        book = None
        for w in word_objects:
            if w.pos == pos_person:
                person = w.token
            if w.pos == pos_book:
                book = w.token
        if person is not None and book is not None:
            if person[0].encode('UTF-8').isalpha():
                e = u"?p :engName '{person}'." \
                    u"?b :title '{book}'." \
                    u"?b :translator ?p.".format(person=person, book=book)  
            else:  
                e = u"?p :chineseName '{person}'." \
                    u"?b :title '{book}'." \
                    u"?b :translator ?p.".format(person=person, book=book)

            sparql = SPARQL_ASK_TEM.format(
                prefix=SPARQL_PREXIX, expression=e)

        return sparql



class PropertyValueSet:
    def __init__(self):
        pass

    @staticmethod
    def return_higher_value():
        return u'>'

    @staticmethod
    def return_lower_value():
        return u'<'
    
    @staticmethod
    def return_equal_value():
        return u'='

    @staticmethod
    def return_title_value():
        return u':title'

    @staticmethod
    def return_rating_value():
        return u':rating'

    @staticmethod
    def return_country_value():
        return u':country'

    @staticmethod
    def return_chinese_name_value():
        return u':chineseName'

    @staticmethod
    def return_english_name_value():
        return u':engName'

    @staticmethod
    def return_writer_value():
        return u':writer'

    @staticmethod
    def return_translator_value():
        return u':translator'

    @staticmethod
    def return_date_value():
        return u':date'

    @staticmethod
    def return_press_value():
        return u':press'

    @staticmethod
    def return_price_value():
        return u':price'

    @staticmethod
    def return_ISBN_value():
        return u':ISBN'

    @staticmethod
    def return_series_value():
        return u':series'

    @staticmethod
    def return_tag_value():
        return u':tag'

# book_basic_keyword_rules = [
#     KeywordRule(condition=book_entity + Star(Any(), greedy=False) + press +
#                 Star(Any(), greedy=False), action=PropertyValueSet.return_press_value),
#     KeywordRule(condition=book_entity + Star(Any(), greedy=False) + rating +
#                 Star(Any(), greedy=False), action=PropertyValueSet.return_rating_value),
#     KeywordRule(condition=book_entity + Star(Any(), greedy=False) + price +
#                 Star(Any(), greedy=False), action=PropertyValueSet.return_price_value),
#     KeywordRule(condition=book_entity + Star(Any(), greedy=False) + date +
#                 Star(Any(), greedy=False), action=PropertyValueSet.return_date_value),
#     KeywordRule(condition=book_entity + Star(Any(), greedy=False) + isbn +
#                 Star(Any(), greedy=False), action=PropertyValueSet.return_ISBN_value),
#     KeywordRule(condition=book_entity + Star(Any(), greedy=False) + series +
#                 Star(Any(), greedy=False), action=PropertyValueSet.return_series_value),
#     KeywordRule(condition=book_entity + Star(Any(), greedy=False) + tag +
#                 Star(Any(), greedy=False), action=PropertyValueSet.return_tag_value)
# ]
book_basic_keyword_rules = [
    KeywordRule(condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + press +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + press +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + press + Star(Any(), greedy=False) + book +
                Star(Any(), greedy=False)), action=PropertyValueSet.return_press_value),
    KeywordRule(condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + rating +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + rating +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + rating + Star(Any(), greedy=False) + book +
                Star(Any(), greedy=False)), action=PropertyValueSet.return_rating_value),
    KeywordRule(condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + price +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + price +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + price + Star(Any(), greedy=False) + book +
                Star(Any(), greedy=False)), action=PropertyValueSet.return_price_value),
    KeywordRule(condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + date +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + date +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + date + Star(Any(), greedy=False) + book +
                Star(Any(), greedy=False)), action=PropertyValueSet.return_date_value),
    KeywordRule(condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + isbn +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + isbn +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + isbn + Star(Any(), greedy=False) + book +
                Star(Any(), greedy=False)), action=PropertyValueSet.return_ISBN_value),
    KeywordRule(condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + series +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + series +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + series + Star(Any(), greedy=False) + book +
                Star(Any(), greedy=False)), action=PropertyValueSet.return_series_value),
    KeywordRule(condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + tag +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + tag +
                Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + tag + Star(Any(), greedy=False) + book +
                Star(Any(), greedy=False)), action=PropertyValueSet.return_tag_value),
]

person_basic_keyword_rules = [
    KeywordRule(condition=person_entity + Star(Any(), greedy=False) + country +
                Star(Any(), greedy=False), action=PropertyValueSet.return_country_value),
    KeywordRule(condition=person_entity + Star(Any(), greedy=False) + english_name +
                Star(Any(), greedy=False), action=PropertyValueSet.return_english_name_value),
    KeywordRule(condition=person_entity + Star(Any(), greedy=False) + chinese_name +
                Star(Any(), greedy=False), action=PropertyValueSet.return_chinese_name_value)
]

compare_keyword_rules = [
    KeywordRule(condition=(Star(Any(), greedy=False) + higher + numeric_entity + Star(Any(), greedy=False) ) | 
                (Star(Any(), greedy=False) + numeric_entity + Star(Any(), greedy=False) + higher + 
                Star(Any(), greedy=False) ), action=PropertyValueSet.return_higher_value),
    KeywordRule(condition=(Star(Any(), greedy=False) + lower + numeric_entity + Star(Any(), greedy=False) ) | 
                (Star(Any(), greedy=False) + numeric_entity + Star(Any(), greedy=False) + lower + 
                Star(Any(), greedy=False) ), action=PropertyValueSet.return_lower_value),
    KeywordRule(condition=Star(Any(), greedy=False) + equal + numeric_entity + Star(Any(), greedy=False), 
                action=PropertyValueSet.return_equal_value),
]

rules = [
    # [人] [写] [?书]
    Rule(condition_num=3, condition=Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + write + 
        Star(Any(), greedy=False) + book + Star(Any(), greedy=False), action=QuestionSet.write_book_question),
    # [人] [翻译] [?书]
    Rule(condition_num=3, condition=Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + translate + 
        Star(Any(), greedy=False) + book + Star(Any(), greedy=False), action=QuestionSet.translate_book_question),
    # [书] [?作者] | [?作者] [书]
    Rule(condition_num=2, condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + writer + 
        Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + writer + Star(Any(), greedy=False) + book_entity + 
        Star(Any(), greedy=False)), action=QuestionSet.has_writer_question),
    # [谁] [写] [书]
    Rule(condition_num=3, condition=Star(Any(), greedy=False) + who + Star(Any(), greedy=False) + write + Star(Any(), greedy=False) +
         book_entity + Star(Any(), greedy=False), action=QuestionSet.has_writer_question),
    # [书] [?译者] | [?译者] [书]     
    Rule(condition_num=2, condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + translator + 
        Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + writer + Star(Any(), greedy=False) + book_entity + 
        Star(Any(), greedy=False)), action=QuestionSet.has_translator_question),
    # [谁] [翻译] [书]
    Rule(condition_num=3, condition=Star(Any(), greedy=False) + who + Star(Any(), greedy=False) + translate + Star(Any(), greedy=False) +
         book_entity + Star(Any(), greedy=False), action=QuestionSet.has_translator_question),
    # [人] [?basic_Info] 
    Rule(condition_num=3, condition=(Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + (what | where) + 
        person_basic + Star(Any(), greedy=False)) |  (Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + 
        person_basic + Star(Any(), greedy=False)), action=QuestionSet.has_basic_person_info_question),
    # [书] [?basic_Info] 
    Rule(condition_num=3, condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + (what | when) + book_basic + 
        Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + book_basic + 
        Star(Any(), greedy=False)), action=QuestionSet.has_basic_book_info_question),
     # [书] [何时] [出版] | [书] [出版] [时间]
    Rule(condition_num=3, condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + when + 
        Star(Any(), greedy=False) + write + Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book_entity + 
        Star(Any(), greedy=False)+ write + Star(Any(), greedy=False) + when + Star(Any(), greedy=False)), 
        action=QuestionSet.has_book_time_question),

    #[作者] [property] [><=] [number] [书]
    Rule(condition_num=5, condition=(Star(Any(), greedy=False) + person_entity + 
        Star(Any(), greedy=False) + book_basic + Star(Any(), greedy=False) + compare + numeric_entity + Star(Any(), greedy=False) + 
        book + Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + person_entity + 
        Star(Any(), greedy=False) + book_basic + Star(Any(), greedy=False) + numeric_entity + Star(Any(), greedy=False) + compare + 
        Star(Any(), greedy=False) + book + Star(Any(), greedy=False)), action=QuestionSet.has_compare_writer_question),
    
     #[作者] [书] [property] [><=] [number] 
    Rule(condition_num=5, condition=(Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + book + 
        Star(Any(), greedy=False) + book_basic + Star(Any(), greedy=False) + compare + numeric_entity +  
        Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + book +
        Star(Any(), greedy=False) + book_basic + Star(Any(), greedy=False) + numeric_entity + Star(Any(), greedy=False) + compare + 
        Star(Any(), greedy=False)), action=QuestionSet.has_compare_writer_question),

    #[作者] [翻译] [property] [><=] [number] [书]
    Rule(condition_num=6, condition=(Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + translate + 
        Star(Any(), greedy=False) + book_basic + Star(Any(), greedy=False) + compare + numeric_entity + Star(Any(), greedy=False) + 
        book + Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + translate + 
        Star(Any(), greedy=False) + book_basic + Star(Any(), greedy=False) + numeric_entity + Star(Any(), greedy=False) + compare + 
        Star(Any(), greedy=False) + book + Star(Any(), greedy=False)), action=QuestionSet.has_compare_translator_question),
    #[作者] [翻译] [书] [property] [><=] [number] 
    Rule(condition_num=6, condition=(Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + translate + 
        Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + book_basic + Star(Any(), greedy=False) + compare + 
        numeric_entity + Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + translate + 
        Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + book_basic + Star(Any(), greedy=False) + numeric_entity + 
        Star(Any(), greedy=False) + compare + Star(Any(), greedy=False)), action=QuestionSet.has_compare_translator_question),
    #[property] [><=] [number] [书]
    Rule(condition_num=4, condition=(Star(Any(), greedy=False) + book_basic + Star(Any(), greedy=False) + compare + numeric_entity + 
        Star(Any(), greedy=False) + book + Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book_basic + 
        Star(Any(), greedy=False) + numeric_entity + Star(Any(), greedy=False) + compare + Star(Any(), greedy=False) + book + 
        Star(Any(), greedy=False)), action=QuestionSet.has_compare_question),
    # [tag] [书] 
    Rule(condition_num=2, condition=(Star(Any(), greedy=False) + tag + Star(Any(), greedy=False) + book + 
        Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + tag + 
        Star(Any(), greedy=False)), action=QuestionSet.has_book_tag_question),
    # [isbn] [number_entity] [书] 
    Rule(condition_num=3, condition=(Star(Any(), greedy=False) + isbn + Star(Any(), greedy=False) + number_entity + Star(Any(), greedy=False) +  book + 
        Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + isbn + 
        Star(Any(), greedy=False) + number_entity + Star(Any(), greedy=False)), action=QuestionSet.has_book_isbn_question),
    # [press] [出版] [书] 
    Rule(condition_num=2, condition=(Star(Any(), greedy=False) + press_entity +  Star(Any(), greedy=False) + press_v + Star(Any(), greedy=False) + book + 
        Star(Any(), greedy=False)) | (Star(Any(), greedy=False) + book + Star(Any(), greedy=False) + press_entity +  Star(Any(), greedy=False) +  press_v + 
        Star(Any(), greedy=False)), action=QuestionSet.has_book_press_question),
    # [作者] [写] [多少] [书]
    Rule(condition_num=4, condition=(Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + write + 
        Star(Any(),greedy=False) + several + Star(Any(),greedy=False) + book  + Star(Any(),greedy=False)) | 
        (Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + write + Star(Any(),greedy=False) + book + 
        Star(Any(),greedy=False) + several  + Star(Any(),greedy=False)), action=QuestionSet.has_write_book_quantity_question),
     # [作者] [翻译] [多少] [书]
    Rule(condition_num=4, condition=(Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + translate + 
        Star(Any(),greedy=False) + several + Star(Any(),greedy=False) + book  + Star(Any(),greedy=False)) | 
        (Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + translate + Star(Any(),greedy=False) + book + 
        Star(Any(),greedy=False) + several  + Star(Any(),greedy=False)), action=QuestionSet.has_translate_book_quantity_question),
    # [书] [人] [翻译] [吗 ]
    Rule(condition_num=4, condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + person_entity +
         translate + Star(Any(), greedy=False) + bool_flag) | (Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + translate +
         book_entity + Star(Any(), greedy=False) + bool_flag) | (Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) +
         book_entity + Star(Any(), greedy=False) + translate + bool_flag), action=QuestionSet.is_tanslator_question),
    # [人]  [书] [作者]  [吗 ]
    Rule(condition_num=4, condition=(Star(Any(), greedy=False) + book_entity + Star(Any(), greedy=False) + person_entity +
        write + Star(Any(), greedy=False) + bool_flag) | (Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + write +
        book_entity + Star(Any(), greedy=False) + bool_flag) | (Star(Any(), greedy=False) + person_entity + Star(Any(), greedy=False) + 
        book_entity + Star(Any(), greedy=False) + writer + bool_flag), action=QuestionSet.is_writer_question),
]


